@@ -23,6 +23,7 @@
 #include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
@@ -191,7 +192,7 @@ path_list_async_new (const char       *uri,
 
 	pli = path_list_data_new ();
 
-	escaped = gnome_vfs_escape_path_string (uri);
+	escaped = escape_uri (uri);
 	if (escaped == NULL) {
 		path_list_data_free (pli);
 		if (f != NULL)
@@ -302,6 +303,71 @@ visit_rc_directory (const gchar *rc_dir,
 	return TRUE;
 }
 
+
+/* Checks all files in ~/RC_DIR and
+ */
+gboolean
+visit_rc_directory_sync (const gchar *rc_dir,
+			 const gchar *rc_ext,
+			 const gchar *dir,
+			 gboolean recursive,
+			 VisitFunc do_something,
+			 gpointer data)
+{
+	gchar *rc_dir_full_path;
+	GList *files, *dirs;
+	GList *scan;
+	gint   prefix_len, ext_len;
+	gchar *prefix;
+	
+	prefix = g_strconcat (g_get_home_dir(), 
+			      "/", 
+			      rc_dir,
+			      NULL);
+	prefix_len = strlen (prefix);
+	rc_dir_full_path = g_strconcat (prefix,
+					dir,
+					NULL);
+	g_free (prefix);
+	ext_len = strlen (rc_ext);
+
+	if (! path_is_dir (rc_dir_full_path)) {
+		g_free (rc_dir_full_path);
+		return FALSE;
+	}
+
+	path_list_new (rc_dir_full_path, &files, &dirs);
+
+	for (scan = files; scan; scan = scan->next) {
+		gchar *rc_file, *real_file;
+
+		rc_file = (gchar*) scan->data;
+		real_file = g_strndup (rc_file + prefix_len, 
+				       strlen (rc_file) - prefix_len - ext_len);
+		if (do_something)
+			(*do_something) (real_file, rc_file, data);
+		
+		g_free (real_file);
+	}
+
+	if (! recursive)
+		return TRUE;
+
+	for (scan = dirs; scan; scan = scan->next) {
+		gchar *sub_dir = (gchar*) scan->data;
+
+		visit_rc_directory_sync (rc_dir,
+					 rc_ext,
+					 sub_dir + prefix_len,
+					 TRUE, 
+					 do_something,
+					 data);
+	}
+
+	return TRUE;
+}
+
+
 /* -- browse_rc_directory_async implementation -- */
 
 typedef struct {
@@ -525,6 +591,49 @@ rmdir_recursive (const gchar *directory)
 /* -- */
 
 
+int
+check_permissions (const char *path,
+		   int         mode) 
+{
+	GnomeVFSFileInfo *info;
+	GnomeVFSResult    result;
+	char             *escaped;
+	int	          perm;
+	
+	info = gnome_vfs_file_info_new ();
+	escaped = escape_uri (path);
+	result = gnome_vfs_get_file_info (escaped, 
+					  info, 
+					  (GNOME_VFS_FILE_INFO_DEFAULT 
+					   | GNOME_VFS_FILE_INFO_FOLLOW_LINKS));
+	
+	if (result != GNOME_VFS_OK) 
+		return FALSE; 
+
+	perm = 7;
+	
+	if ((mode & R_OK) && (info->permissions & 
+			      (GNOME_VFS_PERM_USER_READ | 
+			       GNOME_VFS_PERM_GROUP_READ|
+			       GNOME_VFS_PERM_OTHER_READ))) 
+		perm &= ~R_OK;
+
+	if ((mode & W_OK) && (info->permissions & 
+			      (GNOME_VFS_PERM_USER_WRITE | 
+			       GNOME_VFS_PERM_GROUP_WRITE|
+			       GNOME_VFS_PERM_OTHER_WRITE)))  
+		perm &= ~W_OK;
+	
+	if ((mode & X_OK) && (info->permissions & 
+			      (GNOME_VFS_PERM_USER_EXEC | 
+			       GNOME_VFS_PERM_GROUP_EXEC|
+			       GNOME_VFS_PERM_OTHER_EXEC)))  
+		perm &= ~X_OK;
+
+	return (perm);
+}
+
+
 gboolean
 path_is_file (const char *path)
 {
@@ -533,10 +642,12 @@ path_is_file (const char *path)
 	gboolean          is_file;
 	char             *escaped;
 
-	if (! path || ! *path) return FALSE; 
+	if (! path || ! *path) 
+		return FALSE; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = escape_uri (path);
+
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -552,6 +663,40 @@ path_is_file (const char *path)
 }
 
 
+char *
+escape_uri (const char *uri)
+{
+	const char *start = NULL;
+	const char *uri_no_method;
+	char       *method;
+	char       *epath, *euri;
+
+	if (uri == NULL)
+		return NULL;
+
+	start = strstr (uri, "://");
+	if (start != NULL) {
+		uri_no_method = start + strlen ("://");
+		method = g_strndup (uri, start - uri);
+	} else {
+		uri_no_method = uri;
+		method = NULL;
+	}
+	
+	epath = gnome_vfs_escape_host_and_path_string (uri_no_method);
+	
+	if (method != NULL) {
+		euri = g_strdup_printf ("%s://%s", method, epath);
+		g_free (epath);
+	} else
+		euri = epath;
+
+	g_free (method);
+
+	return euri;
+}
+
+
 gboolean
 path_is_dir (const char *path)
 {
@@ -564,7 +709,8 @@ path_is_dir (const char *path)
 		return FALSE; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = escape_uri (path);
+
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -615,7 +761,7 @@ get_file_size (const char *path)
 	if (! path || ! *path) return 0; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = escape_uri (path);
 	result = gnome_vfs_get_file_info (escaped, 
 					  info,
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -642,7 +788,7 @@ get_file_mtime (const char *path)
 	if (! path || ! *path) return 0; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = escape_uri (path);
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -670,7 +816,7 @@ get_file_ctime (const gchar *path)
 	if (! path || ! *path) return 0; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = escape_uri (path);
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -698,7 +844,7 @@ set_file_mtime (const gchar *path,
 	file_info->mtime = mtime;
 	file_info->atime = mtime;
 
-	escaped_path = gnome_vfs_escape_path_string (path);
+	escaped_path = escape_uri (path);
 	gnome_vfs_set_file_info (escaped_path,
 				 file_info,
 				 GNOME_VFS_SET_FILE_INFO_TIME);
@@ -1071,14 +1217,20 @@ ensure_dir_exists (const char *a_path,
 	char *path;
 	char *p;
 
-	if (! a_path) return FALSE;
+	if (! a_path) 
+		return FALSE;
 
 	if (path_is_dir (a_path))
 		return TRUE;
 
 	path = g_strdup (a_path);
-	p = path;
-	
+
+	p = strstr (path, "://");
+	if (p == NULL)   /* Not a URI */
+		p = path;
+	else  /* Is a URI */
+		p = p + 3;  /* Move p past the :// */
+
 	while (*p != '\0') {
 		p++;
 		if ((*p == '/') || (*p == '\0')) {
@@ -1090,7 +1242,8 @@ ensure_dir_exists (const char *a_path,
 			}
 			
 			if (! path_is_dir (path)) {
-				if (mkdir (path, mode) < 0) {
+				GnomeVFSResult result = gnome_vfs_make_directory (path, mode);
+				if (result != GNOME_VFS_OK) {
 					g_warning ("directory creation failed: %s.", path);
 					g_free (path);
 					return FALSE;
@@ -1118,7 +1271,7 @@ dir_list_filter_and_sort (GList *dir_list,
 	filtered = NULL;
 	scan = dir_list;
 	while (scan) {
-		const gchar *name_only = file_name_from_path (scan->data);
+		const char *name_only = file_name_from_path (scan->data);
 
 		if (! (file_is_hidden (name_only) && ! show_dot_files) 
 		    && (strcmp (name_only, CACHE_DIR) != 0)) {
@@ -1434,9 +1587,14 @@ new_uri_from_path (const char *path)
 	char        *uri_txt;
 	GnomeVFSURI *uri;
 
-	escaped = gnome_vfs_escape_path_string (path);
-	uri_txt = g_strconcat ("file://", escaped, NULL);
+	escaped = escape_uri (path);
+	if (escaped[0] == '/')
+		uri_txt = g_strconcat ("file://", escaped, NULL);
+	else
+		uri_txt = g_strdup (escaped);
+
 	uri = gnome_vfs_uri_new (uri_txt);
+
 	g_free (uri_txt);
 	g_free (escaped);
 
