@@ -29,12 +29,19 @@
 #include "typedefs.h"
 #include "image-viewer.h"
 
+
+#define PREF_DESKTOP_ICON_THEME         "/desktop/gnome/file_views/icon_theme"
+#define PREF_DESKTOP_MENUS_HAVE_TEAROFF "/desktop/gnome/interface/menus_have_tearoff"
+#define PREF_DESKTOP_MENUBAR_DETACHABLE "/desktop/gnome/interface/menubar_detachable"
+#define PREF_DESKTOP_TOOLBAR_DETACHABLE "/desktop/gnome/interface/toolbar_detachable"
+
 #define  PREF_GO_TO_LAST_LOCATION    "/apps/gthumb/general/go_to_last_location"
 #define  PREF_USE_STARTUP_LOCATION   "/apps/gthumb/general/use_startup_location"
 #define  PREF_STARTUP_LOCATION       "/apps/gthumb/general/startup_location"
 #define  PREF_MAX_HISTORY_LENGTH     "/apps/gthumb/general/max_history_length"
 #define  PREF_EDITORS                "/apps/gthumb/general/editors"
 #define  PREF_MIGRATE_DIRECTORIES    "/apps/gthumb/general/migrate_directories"
+#define  PREF_MIGRATE_COMMENT_SYSTEM "/apps/gthumb/general/migrate_comment_system"
 
 #define  PREF_SHOW_HIDDEN_FILES      "/apps/gthumb/browser/show_hidden_files"
 #define  PREF_SHOW_FILENAMES         "/apps/gthumb/browser/show_filenames"
