@@ -27,8 +27,8 @@
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkx.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include "actions.h"
 #include "comments.h"
-#include "commands-impl.h"
 #include "fullscreen.h"
 #include "glib-utils.h"
 #include "gconf-utils.h"
@@ -55,17 +55,13 @@ static FullScreen     *current_fullscreen;
 
 
 static void
-set_command_state (GThumbWindow *window,
-		   char         *cname,
-		   gboolean      setted)
+set_action_active (GThumbWindow *window,
+		   char         *action_name,
+		   gboolean      is_active)
 {
-	char *full_cname = g_strconcat ("/commands/", cname, NULL);
-	char *new_value  = setted ? "1" : "0";
-	bonobo_ui_component_set_prop (window->ui_component, 
-				      full_cname, 
-				      "state", new_value,
-				      NULL);
-	g_free (full_cname);
+	GtkAction *action;
+	action = gtk_action_group_get_action (window->actions, action_name);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), is_active);
 }
 
 
@@ -526,12 +522,12 @@ image_key_press_cb (GtkWidget   *widget,
 
 		/* Toggle animation. */
 	case GDK_g:
-		set_command_state (window, "View_PlayAnimation", ! viewer->play_animation);
+		set_action_active (window, "View_PlayAnimation", ! viewer->play_animation);
 		break;
 
 		/* Step animation. */
 	case GDK_j:
-		view_step_ani_command_impl (NULL, window, NULL);
+		activate_action_view_step_animation (NULL, window);
 		break;
 
 		/* Delete selection. */
@@ -549,9 +545,9 @@ image_key_press_cb (GtkWidget   *widget,
 		/* Delete. */
 
 		if (window->sidebar_content == GTH_SIDEBAR_DIR_LIST)
-			image_delete_command_impl (NULL, window, NULL);
+			activate_action_image_delete (NULL, window);
 		else if (window->sidebar_content == GTH_SIDEBAR_CATALOG_LIST)
-			image_delete_from_catalog_command_impl (NULL, window, NULL);
+			activate_action_edit_remove_from_catalog (NULL, window);
 		break;
 
 		/* Edit comment. */
@@ -566,7 +562,7 @@ image_key_press_cb (GtkWidget   *widget,
 		if (comment_visible)
 			hide_comment_on_image ();
 
-		edit_current_edit_comment_command_impl (NULL, window, NULL);
+		activate_action_edit_edit_comment (NULL, window);
 		break;
 
 		/* Edit categories. */
@@ -581,30 +577,30 @@ image_key_press_cb (GtkWidget   *widget,
 		if (comment_visible)
 			hide_comment_on_image ();
 
-		edit_current_edit_categories_command_impl (NULL, window, NULL);
+		activate_action_edit_edit_categories (NULL, window);
 		break;
 
 		/* Flip image. */
 	case GDK_l:
 	case GDK_L:
-		alter_image_flip_command_impl (NULL, window, NULL);
+		activate_action_alter_image_flip (NULL, window);
 		break;
 
 		/* Rotate. */
 	case GDK_r:
 	case GDK_R:
 	case GDK_bracketright:
-		alter_image_rotate_90_command_impl (NULL, window, NULL);
+		activate_action_alter_image_rotate90 (NULL, window);
 		break;
 
 	case GDK_bracketleft:
-		alter_image_rotate_90_cc_command_impl (NULL, window, NULL);
+		activate_action_alter_image_rotate90cc (NULL, window);
 		break;
 
 		/* Mirror. */
 	case GDK_m:
 	case GDK_M:
-		alter_image_mirror_command_impl (NULL, window, NULL);
+		activate_action_alter_image_mirror (NULL, window);
 		break;
 
 		/* View/Hide comment */
@@ -863,7 +859,6 @@ create_popup_window (void)
 	GtkWidget *frame;
 	GtkWidget *hbox;
 
-
 	popup_window = gtk_window_new (GTK_WINDOW_POPUP);
 
 	frame = gtk_frame_new (NULL);
@@ -894,7 +889,7 @@ create_popup_window (void)
 
 	/* back */
 
-	button = create_button (GTK_STOCK_GO_BACK, _("Back"), FALSE);
+	button = create_button (GTHUMB_STOCK_PREVIOUS_IMAGE, _("Back"), FALSE);
 	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
 	g_signal_connect (G_OBJECT (button),
 			  "clicked",
@@ -903,7 +898,7 @@ create_popup_window (void)
 
 	/* forward */
 
-	button = create_button (GTK_STOCK_GO_FORWARD, _("Forward"), FALSE);
+	button = create_button (GTHUMB_STOCK_NEXT_IMAGE, _("Forward"), FALSE);
 	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
 	g_signal_connect (G_OBJECT (button),
 			  "clicked",
@@ -983,14 +978,8 @@ fullscreen_new (void)
 	if (! fullscreen->wm_state_fullscreen_support) {
 		gtk_widget_destroy (fullscreen->window);
 		fullscreen->window = gtk_window_new (GTK_WINDOW_POPUP);
-		gtk_window_set_default_size (GTK_WINDOW (fullscreen->window), 
-					     gdk_screen_width (), 
-					     gdk_screen_height ());
 	}
 
-	gtk_window_set_wmclass (GTK_WINDOW (fullscreen->window), "",
-				"gthumb_fullscreen");
-
 	fullscreen->related_win = NULL;
 	fullscreen->viewer = NULL;
 
@@ -1031,6 +1020,9 @@ void
 fullscreen_start (FullScreen   *fullscreen,
 		  GThumbWindow *window)
 {
+	int          monitor;
+	GdkRectangle rect;
+
 	g_return_if_fail (fullscreen != NULL);
 
 	if (fullscreen->related_win != NULL)
@@ -1038,6 +1030,11 @@ fullscreen_start (FullScreen   *fullscreen,
 
 	current_fullscreen = fullscreen;
 
+	monitor = gdk_screen_get_monitor_at_window (gdk_screen_get_default (), GTK_WIDGET (window->app)->window);
+	gdk_screen_get_monitor_geometry (gdk_screen_get_default (), monitor, &rect);
+	gtk_window_set_default_size (GTK_WINDOW (fullscreen->window), rect.width, rect.height);
+	gtk_window_move (GTK_WINDOW (fullscreen->window), rect.x, rect.y);
+	
 	gtk_window_set_screen (GTK_WINDOW (fullscreen->window), gtk_widget_get_screen (window->app));
 	gtk_window_present (GTK_WINDOW (fullscreen->window));
 
@@ -1046,9 +1043,10 @@ fullscreen_start (FullScreen   *fullscreen,
 	fullscreen->viewer = window->viewer;
 
 	/* FIXME
-	fullscreen->msg_save_modified_image = eel_gconf_get_boolean (PREF_MSG_SAVE_MODIFIED_IMAGE, TRUE);
-	eel_gconf_set_boolean (PREF_MSG_SAVE_MODIFIED_IMAGE, FALSE);*/
-	
+	   fullscreen->msg_save_modified_image = eel_gconf_get_boolean (PREF_MSG_SAVE_MODIFIED_IMAGE, TRUE);
+	   eel_gconf_set_boolean (PREF_MSG_SAVE_MODIFIED_IMAGE, FALSE);
+	*/
+
 	gtk_window_fullscreen (GTK_WINDOW (fullscreen->window));
 	gtk_window_present (GTK_WINDOW (fullscreen->window));	
 
@@ -1139,7 +1137,7 @@ fullscreen_stop (FullScreen *fullscreen)
 	comment_visible = FALSE;
 
 	/* FIXME
-	eel_gconf_set_boolean (PREF_MSG_SAVE_MODIFIED_IMAGE, fullscreen->msg_save_modified_image);
+	   eel_gconf_set_boolean (PREF_MSG_SAVE_MODIFIED_IMAGE, fullscreen->msg_save_modified_image);
 	*/
 
 	/* release keyboard focus. */ 
@@ -1159,6 +1157,7 @@ fullscreen_stop (FullScreen *fullscreen)
 
 	gtk_widget_reparent (fullscreen->viewer, window->viewer_container);
 	gtk_widget_realize (window->viewer);
+	gtk_widget_show_all (window->viewer);
 	fullscreen->viewer = NULL;
 
 	gtk_widget_set_sensitive (window->app, TRUE);
@@ -1166,6 +1165,7 @@ fullscreen_stop (FullScreen *fullscreen)
 
 	/* restore widgets visiblity */
 
+	gtk_widget_hide (window->progress);
 	window_set_preview_content (window, window->preview_content);
 	if (! window->image_pane_visible)
 		window_hide_image_pane (window);
