@@ -27,9 +27,6 @@
 #include <stdlib.h>
 #include <time.h>
 #include <glib.h>
-#include <exif-data.h>
-#include <exif-content.h>
-#include <exif-entry.h>
 #include "gth-exif-utils.h"
 
 
